# Copyright 2012 GRNET S.A. All rights reserved.
#
# Redistribution and use in source and binary forms, with or
# without modification, are permitted provided that the following
# conditions are met:
#
#   1. Redistributions of source code must retain the above
#      copyright notice, this list of conditions and the following
#      disclaimer.
#
#   2. Redistributions in binary form must reproduce the above
#      copyright notice, this list of conditions and the following
#      disclaimer in the documentation and/or other materials
#      provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY GRNET S.A. ``AS IS'' AND ANY EXPRESS
# OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
# PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL GRNET S.A OR
# CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
# USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
# AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
# The views and conclusions contained in the software and
# documentation are those of the authors and should not be
# interpreted as representing official policies, either expressed
# or implied, of GRNET S.A.


from django.db import models
from django.conf import settings
from django.contrib.sites import models as sites_models

from feincms.module.medialibrary.fields import MediaFileForeignKey
from feincms.module.medialibrary.models import MediaFile


class Application(models.Model):
    code = models.CharField('Identifier', max_length=100, null=False, blank=False,
            help_text="Just a codename of the application, to be used in "\
                    "several places where no free text is allowed"\
                    "(e.g. urls, paths, etc)")
    title = models.CharField(max_length=255, null=False, blank=False,
            help_text="The title of the application")

    logo = MediaFileForeignKey(MediaFile, blank=True, null=True)
    site = models.ForeignKey(sites_models.Site)
    app_url = models.URLField(help_text="The url of the application UI (not "\
            "the cms", verify_exists=False, blank=True, null=True)

    linked_in_username = models.CharField(max_length=255, blank=True)
    twitter_username = models.CharField(max_length=255, blank=True)
    facebook_username = models.CharField(max_length=255, blank=True)

    show_twitter_feed_on_top = models.BooleanField(default=False)
    extra_styles = models.TextField(default="", blank=True)

    footer_top = models.TextField(default="", blank=True)
    footer_bottom = models.TextField(default="", blank=True)

    @classmethod
    def current(cls):
        return cls.objects.get(site__pk=settings.SITE_ID)

    def __unicode__(self):
        return self.title

# hook for feincms configuration, is this appropriate place ??? who knows
from cloudcms.cms import *

